//----------------------------------------------------------------------------
//	HEADER:	App.h
//	AUTHOR:	Ian Taylor
//	NOTES:	Curling game application class
//----------------------------------------------------------------------------

#ifndef TEA_APP_H
#define TEA_APP_H
#ifdef TEA_DEBUG_INCLUDES
#pragma message("#include " __FILE__)
#endif

#ifndef TEA_CONFIG_H
#include "Config.h"
#endif
#ifndef TEA_RENDER_H
#include "Render.h"
#endif
#ifndef TEA_SOUND_H
#include "Sound.h"
#endif
#ifndef TEA_GAME_H
#include "Game.h"
#endif

class Application
{
	private:

		// Private data
		Configuration m_config;		// The current configuration
		Renderer* m_renderer;		// The current renderer, if any
		Sound m_sound;				// Sound interface
		TEA::Random m_random;		// The random number generator
		Game m_game;				// The current game

	public:

		// No construction, just initialisation
		bool initialise(void);
		void shutdown(void);

		// Synchronous events
		void onIdle(DialogMain& main);
		void onWakeup(DialogMain& main);
		void onCanvasPaint(void)
		{
			// Need to paint the canvas
			MONITOR(Application::onCanvasPaint)
			if (m_renderer == NULL)
				Renderer::defaultPaint(hcanvas());
			else
				m_renderer->windowPaint(hcanvas());
		}
		void onCanvasChange(Renderer::Change what)
		{
			// Need to change the canvas
			MONITOR(Application::onCanvasChange)
			if (m_renderer != NULL)
				if (m_renderer->windowChanged(hcanvas(), what))
					m_renderer->sceneUpdate(m_game.view);
		}
		void onCanvasClick(DialogMain& main, int mx, int my);
		void onCanvasThrow(DialogMain& main);
		void onCanvasZoom(real zoom);
		int onDialogConfig(void);
		bool onDialogViolation(const char* violation = NULL);
		bool onDialogAbout(void);

		// Methods
		Game& newGame(void)
		{
			// Start a new game
			m_game.newGame(m_config, m_random);
			ASSERT(m_renderer != NULL);
			m_renderer->sceneUpdate(m_game.view);
			VALIDATE(m_game);
			return m_game;
		}
		void newRenderer(Renderer::Type type);
		void fatalError(const TEA::Error& error);
		int fatalError(const char* function) { fatalError(TEA::Error(function)); return 0; }
		void silence(void);
		bool enableSound(bool enable);
		bool enableQuick(bool enable);

		// Properties
		Window hwnd(void) const;
		Window hcanvas(void) const;
		void canvasSize(int& sx, int& sy) const;
		const Game& game(void) const { return m_game; }

		// Debugging
		bool isValid(void) const
		{
			return TEA::Memory::isValidRead(this);
		}
};

#endif
