//----------------------------------------------------------------------------
//	SOURCE:	Collide.cpp
//	AUTHOR:	Ian Taylor
//	NOTES:	Curling collision class
//----------------------------------------------------------------------------

#include "Curling.h"
#include "Collide.h"

TEA_NAMESPACE_USE

bool Collide::hitGutterCurl(real gx, real curl, real& t) const
{
	// Determine when the curling stone hits a side gutter
	MONITOR(Collide::hitGutterCurl)
	ASSERT(ox == 0);
	ASSERT(vy > 0);
	real v = vx / vy;
	real a1 = v * v + REAL(1.0);
	real b2 = v * (curl - gx);
	real c1 = gx * (gx - curl * REAL(2.0));
	real q4 = b2 * b2 - a1 * c1;
	if (q4 < REAL(0.0))
		return false;
	real q2 = Math::sqrt(q4);
	real y1 = (q2 + b2) / -a1;
	if (y1 > REAL(0.0))
		t = y1 / vy;
	else
	{
		// Try the later solution
		real y2 = (q2 - b2) / a1;
		if (y2 > REAL(0.0))
			t = y2 / vy;
		else
			return false;
	}
	return true;
}
