//----------------------------------------------------------------------------
//	HEADER:	Collide.h
//	AUTHOR:	Ian Taylor
//	NOTES:	Curling collision class
//----------------------------------------------------------------------------

#ifndef TEA_COLLIDE_H
#define TEA_COLLIDE_H
#ifdef TEA_DEBUG_INCLUDES
#pragma message("#include " __FILE__)
#endif

struct Collide
{
	// Our stone parameters
	real ox, oy;
	real vx, vy;

	bool hitStone(real sx, real sy, real& t) const
	{
		// Determine when the stone hits another one
		MONITOR(Collide::hitStone)
		real dx = ox - sx;
		real dy = oy - sy;
		real b2 = vx * dx + vy * dy;
		if (b2 >= REAL(0.0))
			return false;
		real a1 = vx * vx + vy * vy;
		ASSERT(a1 > 0);
		real c1 = dx * dx + dy * dy - REAL(CURLING_STONE_DIAMETER) * REAL(CURLING_STONE_DIAMETER);
		ASSERT(c1 > 0);
		real q4 = b2 * b2 - a1 * c1;
		if (q4 < REAL(0.0))
			return false;
		t = (b2 + TEA::Math::sqrt(q4)) / -a1;
		ASSERT(t >= 0);
		return true;
	}
	bool hitGutter(real gx, real& t) const
	{
		// Determine when the stone hits a side gutter
		MONITOR(Collide::hitGutter)
		if (TEA::Math::isNearZero(vx))
			return false;
		real dx = gx - ox;
		t = dx / vx;
		return t >= REAL(0.0);
	}
	bool hitEnd(real gy, real& t) const
	{
		// Determine when the stone hits the end
		MONITOR(Collide::hitEnd)
		if (vy < TEA::Math::Epsilon)
			return false;
		t = (gy - oy) / vy;
		ASSERT(t >= 0);
		return true;
	}
	bool hitStoneCurl(real sx, real sy, real curl, real& t) const;
	bool hitGutterCurl(real gx, real curl, real& t) const;
	bool hitEndCurl(real gy, real, real& t) const { return hitEnd(gy, t); }
};

#endif
