//----------------------------------------------------------------------------
//	HEADER:	Curling.h
//	AUTHOR:	Ian Taylor
//	NOTES:	Main header file for Curling game
//----------------------------------------------------------------------------

// Show the progress of the includes
#ifdef TEA_DEBUG_INCLUDES
#pragma message("#include " __FILE__)
#endif

// The Tea sub-system
#include "Tea.h"

// Bring the useful types into scope
using TEA::real;
using TEA::System::Window;

// Don't update the screen more than once in this number of seconds
#define CURLING_UPDATE_THROTTLE		0.001

// Constants (metres)
#define CURLING_STONE_DIAMETER		  0.2911
#define CURLING_RINK_LENGTH			 47.4980
#define CURLING_RINK_WIDTH			  4.3180
#define CURLING_RINK_END			  3.0000	// Actually 6.4770
#define CURLING_RINK_BACK			  1.8288
#define CURLING_RINK_HOG			 -6.4008
#define CURLING_RINK_STRAIGHTEN		 -7.1000
#define CURLING_RINK_RELEASE		-37.0000
#define CURLING_RINK_HACK			-38.4048
#define CURLING_GUTTER_WIDTH		  0.4000
#define CURLING_HOARDINGS_HEIGHT	  0.3000
#define CURLING_NUDGE				  0.0010
#define CURLING_RANDOM_ICE			  0.1000

// Constants (pixels)
#define CURLING_TEXTURE_SIZE		1024
#define CURLING_LINE_WIDTH			4

// Constants (radians)
#define CURLING_COT_FOV_BY_2		2.0

// Constants (throw parameters)
#define CURLING_SPIN_MAX			0.01
#define CURLING_SPIN_MIN			-CURLING_SPIN_MAX
#define CURLING_POWER_MAX			(-CURLING_POWER_MIN - CURLING_RINK_RELEASE * 2)
#define CURLING_POWER_MIN			30
#define CURLING_AIM_MAX				0.2
#define CURLING_AIM_MIN				-CURLING_AIM_MAX
#define CURLING_INACCURACY_MAX		1.0
#define CURLING_VISUAL_SPIN			-2

// Constant (deceleration in metres per second per second)
#define CURLING_DECELERATION		0.1

// Constant (coefficient of restitution)
#define CURLING_RESTITUTION			0.3

// Constant (time in seconds of the drawback sequence)
#define CURLING_DRAWBACK_TIME		1.3

// Constant (power factor for camera lag)
#define CURLING_CAMERA_LAG			1.5

// The number of seconds the quick camera jumps to before the first event
#define CURLING_QUICK_CAMERA		1
