//----------------------------------------------------------------------------
//	HEADER:	Midi.h
//	AUTHOR:	Ian Taylor
//	NOTES:	Curling game MIDI interface
//----------------------------------------------------------------------------

#ifndef TEA_MIDI_H
#define TEA_MIDI_H
#ifdef TEA_DEBUG_INCLUDES
#pragma message("#include " __FILE__)
#endif

// MIDI message creators
#define MIDI_MSG(s, a, b)		((s) | ((a) << 8) | ((b) << 16))
#define MIDI_MSG_OFF(c, p, v)	MIDI_MSG(0x7F + (c), (p), (v))
#define MIDI_MSG_ON(c, p, v)	MIDI_MSG(0x8F + (c), (p), (v))
#define MIDI_MSG_AFTER(c, p, v)	MIDI_MSG(0x9F + (c), (p), (v))
#define MIDI_MSG_CTRL(c, i, v)	MIDI_MSG(0xAF + (c), (i), (v))
#define MIDI_MSG_PROG(c, p)		MIDI_MSG(0xBF + (c), (p), 0)
#define MIDI_MSG_POLY(c, p)		MIDI_MSG(0xCF + (c), (p), 0)
#define MIDI_MSG_PITCH(c, p)	MIDI_MSG(0xDF + (c), (p) % 128, (p) / 128)
#define MIDI_MSG_VOLUME(c, v)	MIDI_MSG_CTRL((c), 7, v)

// Singleton
class MidiDevice
{
	public:

		// Midi messages
		typedef TEA::uint32 Msg;

		// Initialisation/shutdown
		static bool initialise(void);
		static void shutdown(void);

		// Place in the queue
		static bool playAsync(const Msg* msg, int count);
		static bool playAsync(Msg msg) { return playAsync(&msg, 1); }
		static bool playSync(Msg msg);
		static bool playOutOfBounds(Msg msg);

		// Quieten all voices and empty the queue
		static bool reset(void);
};

#endif
