//----------------------------------------------------------------------------
//	HEADER:	Render.h
//	AUTHOR:	Ian Taylor
//	NOTES:	Curling game renderer interface
//----------------------------------------------------------------------------

#ifndef TEA_RENDER_H
#define TEA_RENDER_H
#ifdef TEA_DEBUG_INCLUDES
#pragma message("#include " __FILE__)
#endif

TEA_HANDLE(HDC)
struct GameView;

class TEA_NOVTABLE Renderer
{
	public:

		// Texture creation
		enum
		{
			TextureWidth = CURLING_TEXTURE_SIZE,
			TextureHeight = CURLING_TEXTURE_SIZE
		};
		static long textureLoad(HDC hdc, const char* message);

		// Types
		enum Type
		{
			DirectX7
		};

		// Change flags
		enum Change
		{
			ChangePosition,
			ChangeSize,
			ChangeMode
		};

		// Factories
		static Renderer* create(Type type, Window window);
		static Renderer* createDirectX7(Window window);

		// Helpers
		static void defaultPaint(Window window) { errorPaint(window, NULL); }
		static void errorPaint(Window window, const char* message);

		// Methods
		virtual bool windowReady(void) = 0;
		virtual bool windowChanged(Window window, Change what) = 0;
		virtual void windowPaint(Window window) = 0;
		virtual void sceneUpdate(const GameView& view) = 0;
		virtual void destroy(void) = 0;

		// In-place construction
		void* operator new(size_t, void* in_situ)
		{
			// This is a kludge to make sure the vtable point is initialised
			ASSERT(in_situ != NULL);
			return in_situ;
		}
		void operator delete(void*, void*)
		{
			// In-place destruction to keep the compiler happy
		}

		// Debugging
		bool isValid(void) const
		{
			return TEA::Memory::isValidRead(this);
		}
};

#endif
