//----------------------------------------------------------------------------
//	SOURCE:	TCRT.cpp
//	AUTHOR:	Ian Taylor
//	NOTES:	Tea cutdown CRT library
//----------------------------------------------------------------------------

#define WIN32_LEAN_AND_MEAN
#define STRICT
#include <windows.h>

// This is the only interest bit here, honest
void WINAPI TeaEntryPoint(void)
{
	// Jumps here from the operating system
	HINSTANCE module = ::GetModuleHandle(NULL);
	int retcode = WinMain(module, NULL, NULL, SW_SHOWDEFAULT);
	::ExitProcess(retcode);
}

// Naked assembler routines with standard names mangling (prefix '_')
#define CRTAPI_(rettype)	extern "C" rettype __declspec(naked) __cdecl
#define CRTAPI				CRTAPI_(void)

// Win32 uses 4096-byte pages
#define PAGESIZE	0x1000

CRTAPI main(void)
{
	// Only needed for Standard Edition C++ (!?!)
}

// Disable warning about inaccuracy on some Pentium/MSDEV configurations
#pragma warning(disable: 4725)

CRTAPI _CIsqrt(void)
{
	__asm
	{
		fsqrt
		ret
	}
}

CRTAPI _CIsin(void)
{
	__asm
	{
		fsin
		ret
	}
}

CRTAPI _CIcos(void)
{
	__asm
	{
		fcos
		ret
	}
}

CRTAPI _CItan(void)
{
	__asm
	{
		fptan
		ret
	}
}

CRTAPI _CIatan2(void)
{
	__asm
	{
		fpatan
		ret
	}
}

CRTAPI _CIpow(void)
{
	// This is a re-implementation of Microsoft's undocumented function
	__asm
	{
		fxch
		fyl2x
		fld		st(0)
		frndint
		fsubr	st(1), st
		fxch	st(1)
		fchs
		f2xm1
		fld1
		faddp	st(1), st
		fscale
		fstp	st(1)
		ret
	}
}

CRTAPI _chkstk(void)
{
	// As per Microsoft's but with the optimisation suggested in their source(!)
	__asm
	{
		// Extends the stack but read memory every page to ensure the VMM gets a look-in
		cmp		eax, PAGESIZE
		jb		chkstk_single
		push	ecx
		lea		ecx, [esp]+8

chkstk_next:

		// Standard probe from Microsoft
		sub		ecx, PAGESIZE
		sub		eax, PAGESIZE
		test	[ecx], eax
		cmp		eax, PAGESIZE
		jae		chkstk_next
		sub		ecx, eax
		mov		eax, esp
		test	[ecx], eax
		mov		esp, ecx
		mov		ecx, [eax]
		mov		eax, [eax+4]
		push	eax
		ret

chkstk_single:

		// This is the optimisation Microsoft never implemented (saves four instructions)
		sub		esp, eax
		add		eax, esp
		test	[esp+4], eax
		mov		eax, [eax]
		mov		[esp], eax
		ret
	}
}

CRTAPI_(int) _purecall(void)
{
	// This is a placeholder for a pure virtual function. Never happens(?)
	__asm int 3;
}

// Snore...
extern "C" const int _fltused = 0;
