//----------------------------------------------------------------------------
//	HEADER:	TCom.h
//	AUTHOR:	Ian Taylor
//	NOTES:	Tea COM synthesis
//----------------------------------------------------------------------------

#ifndef TEA_COM_H
#define TEA_COM_H
#ifdef TEA_DEBUG_INCLUDES
#pragma message("#include " __FILE__)
#endif

TEA_NAMESPACE_BEGIN

template<typename T>
class ComPtr
{
	public:

		// Types
		typedef T Type;

	private:

		// Data members
		Type* m_ptr;

		// Implementation
		void _addRef(void) { if (m_ptr != NULL) m_ptr->AddRef(); }
		void _release(void) { if (m_ptr != NULL) m_ptr->Release(); }

	public:

		// Construction/destruction
		ComPtr(void) : m_ptr(NULL) {}
		explicit ComPtr(Type* ptr) : m_ptr(ptr) { _addRef(); }
		explicit ComPtr(const ComPtr<T>& ptr) : m_ptr(ptr.m_ptr) { _addRef(); }
		~ComPtr(void) { _release(); }

		// Methods
		void release(void) { if (m_ptr != NULL) { Type* t = m_ptr; m_ptr = NULL; t->Release(); } }

		// Conversions
		Type* operator->(void) const { ASSERT(m_ptr != NULL); return m_ptr; }
		operator T*(void) const { return m_ptr; }
		Type& operator*(void) const { ASSERT(m_ptr != NULL); return *m_ptr; }
		Type** operator&(void) { ASSERT(m_ptr == NULL); return &m_ptr; }

		// Comparisons
		bool operator==(T* ptr) const { return m_ptr == ptr; }
		bool operator!=(T* ptr) const { return m_ptr != ptr; }

		// Debugging
		bool isValid(void) const
		{
			return Memory::isValidRead(this) && ((m_ptr == NULL) || Memory::isValidRead(m_ptr));
		}
};

TEA_NAMESPACE_END

#endif
