//----------------------------------------------------------------------------
//	SOURCE:	TError.cpp
//	AUTHOR:	Ian Taylor
//	NOTES:	Tea error class
//----------------------------------------------------------------------------

#include "Tea.h"
#include "TFormat.h"

TEA_NAMESPACE_USE

void Error::report(System::Window window, const char* caption) const
{
	MONITOR(Error::report)

	// Report an error
	char buffer[1024];
	format(buffer, dimof(buffer));
	System::errorReport(window, caption, buffer);
	return;
}

void Error::format(char* buffer, size_t size) const
{
	MONITOR(Error::format)

	// Format this error
	VALIDATE(*this);
	ASSERT(size > 0);
	ASSERT(Memory::isValidWrite(buffer, size));
	const char* reason = m_reason;
	char sysmsg[1024];
	if (reason == NULL)
	{
		if (System::errorFormat(sysmsg, dimof(sysmsg), m_errcode))
			reason = sysmsg;
		else
			reason = "Invalid error";
	}
	if (m_function != NULL)
		if (Format::concat(buffer, size, "Fatal error in '"))
			if (Format::concat(buffer, size, m_function))
				Format::concat(buffer, size, "':\r\n\r\n");
	Format::concat(buffer, size, reason);
	return;
}