//----------------------------------------------------------------------------
//	HEADER:	TError.h
//	AUTHOR:	Ian Taylor
//	NOTES:	Tea errors
//----------------------------------------------------------------------------

#ifndef TEA_ERROR_H
#define TEA_ERROR_H
#ifdef TEA_DEBUG_INCLUDES
#pragma message("#include " __FILE__)
#endif

TEA_NAMESPACE_BEGIN

class Error
{
	private:

		// Data members
		const char* m_function;
		const char* m_reason;
		long m_errcode;

	public:

		// Construction
		Error(long errcode = 0)
			: m_function(NULL), m_reason(NULL), m_errcode(errcode) {}
		Error(const char* function, long errcode = System::errorGetLast())
			: m_function(function), m_reason(NULL), m_errcode(errcode) {}
		Error(const char* function, const char* reason, long errcode = 0)
			: m_function(function), m_reason(reason), m_errcode(errcode) {}

		// Methods
		void set(long errcode)
		{
			// Just an error code was supplied
			m_function = NULL;
			m_reason = NULL;
			m_errcode = errcode;
		}
		void set(const char* function, long errcode)
		{
			// A function name and an error code were supplied
			m_function = function;
			m_reason = NULL;
			m_errcode = errcode;
		}
		void set(const char* function, const char* reason, long errcode = 0)
		{
			// Lots of information supplied
			m_function = function;
			m_reason = reason;
			m_errcode = errcode;
		}
		void format(char* buffer, size_t size) const;
		void report(System::Window window, const char* caption) const;

		// Properties
		long errcode(void) const { return m_errcode; }

		// Debugging
		bool isValid(void) const
		{
			return Memory::isValidRead(this);
		}
};

TEA_NAMESPACE_END

#endif
