//----------------------------------------------------------------------------
//	HEADER:	TFormat.h
//	AUTHOR:	Ian Taylor
//	NOTES:	String formatting
//----------------------------------------------------------------------------

#ifndef TEA_FORMAT_H
#define TEA_FORMAT_H
#ifdef TEA_DEBUG_INCLUDES
#pragma message("#include " __FILE__)
#endif

TEA_NAMESPACE_BEGIN

namespace Format
{
	// Character buffer manipulations
	inline bool scan(char*& src, size_t& size, char ch = '\0')
	{
		// Find the first occurence of a character in a text buffer
		ASSERT(size > 0);
		ASSERT(Memory::isValidWrite(src, size));
		while (*src != ch)
		{
			src++;
			if (--size == 0)
				return false;
		}
		return true;
	}
	inline bool concat(char*& dst, size_t& size, const char* src)
	{
		// Concatenate one text buffer on to another
		ASSERT(size > 0);
		ASSERT(Memory::isValidWrite(dst, size));
		ASSERT(Memory::isValidRead(src));
		while (*src != '\0')
		{
			if (size <= 1)
			{
				// There's no space for more
				*dst = '\0';
				return false;
			}
			*(dst++) = *(src++);
			size--;
		}
		*dst = '\0';
		return true;
	}

	// Integers to text
	template<typename T>
	inline char* fromUnsigned(T value, char* buffer, size_t size)
	{
		// Convert an unsigned integer to text
		ASSERT(value >= 0);
		ASSERT(buffer != NULL);
		ASSERT(size >= 2);
		buffer[--size] = '\0';
		do
		{
			ASSERT(size > 0);
			buffer[--size] = char(value % 10 + '0');
			value /= 10;
		}
		while (value > 0);
		return &buffer[size];
	}
	template<typename T>
	inline char* fromSigned(T value, char* buffer, size_t size)
	{
		// Convert a signed integer to text
		bool negative = (value < 0);
		if (negative)
			value = -value;
		char* sign = fromUnsigned(value, buffer, size);
		if (negative)
		{
			ASSERT(sign > buffer);
			*--sign = '-';
		}
		return sign;
	}
	template<typename T>
	inline char* fromPadded(T value, char* buffer, size_t size, char pad = ' ')
	{
		// Convert an integer to fixed-length text
		ASSERT(value >= 0);
		ASSERT(buffer != NULL);
		ASSERT(size >= 2);
		buffer[--size] = '\0';
		do
		{
			ASSERT(size > 0);
			buffer[--size] = char(value % 10 + '0');
			value /= 10;
		}
		while (value > 0);
		while (size > 0)
			buffer[--size] = pad;
		return buffer;
	}
}

TEA_NAMESPACE_END

#endif
