//----------------------------------------------------------------------------
//	HEADER:	TMemory.h
//	AUTHOR:	Ian Taylor
//	NOTES:	Memory checks
//----------------------------------------------------------------------------

#ifndef TEA_MEMORY_H
#define TEA_MEMORY_H
#ifdef TEA_DEBUG_INCLUDES
#pragma message("#include " __FILE__)
#endif

TEA_NAMESPACE_BEGIN

namespace Memory
{
	// Memory block operations
	inline bool isValidReadPtr(const void* ptr, size_t size)
	{
		// Determine if it points to readable memory
		MONITOR(Memory::isValidReadPtr)
		ASSERT(ptr != NULL);
		ASSERT(size > 0);
		return System::memoryIsRead(ptr, size);
	}
	inline bool isValidWritePtr(void* ptr, size_t size)
	{
		// Determine if it points to writable memory
		MONITOR(Memory::isValidWritePtr)
		ASSERT(ptr != NULL);
		ASSERT(size > 0);
		return System::memoryIsWrite(ptr, size);
	}

	// Strongly-typed operations
	template<typename T>
	inline void swap(T& a, T& b)
	{
		// Swap two 'things'
		MONITOR(Memory::swap<>)
		ASSERT(&a != &b);
		T t(a);
		a = b;
		b = t;
	}
	template<typename T>
	inline void zero(T& a)
	{
		// Zero out a 'thing'
		MONITOR(Memory::zero<>)
		::memset(&a, 0, sizeof(a));
	}
	template<typename T>
	inline bool isValidRead(const T* ptr, size_t size = 1)
	{
		// Determine if it points to readable memory
		MONITOR(Memory::isValidRead<>)
		ASSERT(size > 0);
		return (ptr != NULL) && System::memoryIsRead(ptr, size * sizeof(*ptr));
	}
	template<typename T>
	inline bool isValidWrite(T* ptr, size_t size = 1)
	{
		// Determine if it points to writable memory
		MONITOR(Memory::isValidWrite<>)
		ASSERT(size > 0);
		return (ptr != NULL) && System::memoryIsWrite(ptr, size * sizeof(*ptr));
	}
};

TEA_NAMESPACE_END

#endif
