//----------------------------------------------------------------------------
//	HEADER:	TMonitor.h
//	AUTHOR:	Ian Taylor
//	NOTES:	Monitoring helpers
//----------------------------------------------------------------------------

#ifndef TEA_MONITOR_H
#define TEA_MONITOR_H
#ifdef TEA_DEBUG_INCLUDES
#pragma message("#include " __FILE__)
#endif

// Unless told otherwise, the monitor level is the same as the debug level
#ifndef TEA_MONITOR_LEVEL
#define TEA_MONITOR_LEVEL	0
#endif

#if TEA_MONITOR_LEVEL > 0

TEA_NAMESPACE_BEGIN

class Monitor
{
	public:

		// Usually 64-bits
		typedef System::Timer Time;

		// This is a simpler wrapper around performance counters
		static void assignClock(Time& elapsed);

		// One-time initialisation/shutdown
		static void initialise(void);
		static void shutdown(Time total);

		// Logging
		static void logToFile(const char* filename, const char* mode = "a");

		// These are instances of functions (static)
		struct Function
		{
			// Data members
			const char* name;
			const char* file;
			int line;
			int calls;
			Time in_function;
			Time in_children;
			Function* next;
		};

	private:

		// Private data
		Function& m_function;
		Monitor* m_caller;
		Time m_inFunction;
		Time m_inChildren;

		// Disable assignment
		Monitor& operator=(const Monitor&);

	public:

		// Construction/destruction
		Monitor(Function& function);
		~Monitor(void);

		// Methods
		void addChildren(Time time) { m_inChildren += time; }

		// Helpers
		static void output(const char* text);
		static void outputTime(const char* text, Time time);
		static void outputLine(void);
};

TEA_NAMESPACE_END

// Monitor helper
#define MONITOR(function)																							\
	static TEA::Monitor::Function _monitor_function = { #function, __FILE__, __LINE__, NULL };	\
	TEA::Monitor _monitor(_monitor_function);

#else

// No monitoring
#define MONITOR(function)

#endif

#endif
