//----------------------------------------------------------------------------
//	HEADER:	TRandom.h
//	AUTHOR:	Ian Taylor
//	NOTES:	Tea random numbers
//----------------------------------------------------------------------------

#ifndef TEA_RANDOM_H
#define TEA_RANDOM_H
#ifdef TEA_DEBUG_INCLUDES
#pragma message("#include " __FILE__)
#endif

TEA_NAMESPACE_BEGIN

// Random number generator (courtesy of Tom Hyer, Esq., All Rights Reserved)
class Random
{
	private:

		// Private data
		real m_x[97];
		uint32 m_ix1;
		uint32 m_ix2;
		uint32 m_ix3;

	public:

		// Methods
		void reset(int seed = System::randomSeed())
		{
			// Reset to a new seed
			MONITOR(Random::reset)
			m_ix1 = (54773 - seed) % 259200;
			m_ix1 = (7141 * m_ix1 + 54773) % 259200;
			m_ix2 = m_ix1 % 134456;
			m_ix1 = (7141 * m_ix1 + 54773) % 259200;
			m_ix3 = m_ix1 % 243000;
			for (int index = 0; index < dimof(m_x); index++)
			{
				m_ix1 = (7141 * m_ix1 + 54773) % 259200;
				m_ix2 = (8121 * m_ix2 + 28411) % 134456;
				m_x[index] = (m_ix1 + m_ix2 / real(134456)) / real(259200);
			}
		}
		int get(int imod)
		{
			// Fetch a capped integer from the generate (make sure we use the MSBs)
			MONITOR(Random::get)
			VALIDATE(*this);
			ASSERT(imod > 1);
			int out = static_cast<int>(System::mathTruncate(uniform() * imod));
			ASSERT((out >= 0) && (out < imod));
			return out;
		}
		real uniform(void)
		{
			// Fetch the next number from the generator
			MONITOR(Random::uniform)
			VALIDATE(*this);
			m_ix1 = (7141 * m_ix1 + 54773) % 259200;
			m_ix2 = (8121 * m_ix2 + 28411) % 134456;
			m_ix3 = (4561 * m_ix3 + 51349) % 243000;
			int index = (97 * m_ix3) / 243000;
			real out = m_x[index];
			m_x[index] = (m_ix1 + m_ix2 / real(134456)) / real(259200);
			ASSERT((out >= 0) && (out < 1));
			return out;
		}
		real uniform(real lbound, real ubound)
		{
			// Fetch a floating-point number between two bounds inclusive
			MONITOR(Random::uniform)
			VALIDATE(*this);
			ASSERT(lbound <= ubound);
			real out = uniform() * (ubound - lbound) + lbound;
			ASSERT((out >= lbound) && (out <= ubound));
			return out;
		}
		real bell(int samples = 7)
		{
			// Fetch a random from a bell-shaped distribution between -1 and +1 inclusive
			MONITOR(Random::bell)
			VALIDATE(*this);
			ASSERT(samples > 0);
			real out = uniform();
			for (int i = 1; i < samples; i++)
				out += uniform();
			out = out * 2 / samples - 1;
			ASSERT((out >= -1) && (out <= 1));
			return out;
		}
		real bell(real lbound, real ubound, int samples = 7)
		{
			// Fetch a random from a bell-shaped distribution between two bounds inclusive
			MONITOR(Random::bell)
			VALIDATE(*this);
			ASSERT(lbound <= ubound);
			ASSERT(samples > 0);
			real out = bell(samples) * (ubound - lbound) / 2 + lbound;
			ASSERT((out >= lbound) && (out <= ubound));
			return out;
		}

		// Debugging
		bool isValid(void) const
		{
			return TEA::Memory::isValidRead(this) &&
				   (m_ix1 >= 0) && (m_ix1 < 259200) &&
				   (m_ix2 >= 0) && (m_ix2 < 134456) &&
				   (m_ix3 >= 0) && (m_ix3 < 243000);
		}
};

TEA_NAMESPACE_END

#endif
