//----------------------------------------------------------------------------
//	HEADER:	TSystem.h
//	AUTHOR:	Ian Taylor
//	NOTES:	System-specific definitions
//----------------------------------------------------------------------------

#ifndef TEA_SYSTEM_H
#define TEA_SYSTEM_H
#ifdef TEA_DEBUG_INCLUDES
#pragma message("#include " __FILE__)
#endif

// Namespace definitions
#ifdef TEA_NO_NAMESPACE

// Don't use a namespace
#define TEA
#define TEA_NAMESPACE_BEGIN
#define TEA_NAMESPACE_END
#define TEA_NAMESPACE_USE

#else

// The default namespace is 'Tea'
#ifndef TEA_NAMESPACE
#define TEA_NAMESPACE			Tea
#endif
#define TEA						::TEA_NAMESPACE
#define TEA_NAMESPACE_BEGIN		namespace TEA_NAMESPACE {
#define TEA_NAMESPACE_END		};
#define TEA_NAMESPACE_USE		using namespace TEA_NAMESPACE;

#endif

// Platform-specific includes (first time)
#ifdef WIN32
#include "TWin32.h"
#endif

// Get a good definition for NULL
#ifndef NULL
#define NULL					0
#endif

// Useful anywhere
#define dimof(x)				(sizeof(x) / sizeof(*(x)))

TEA_NAMESPACE_BEGIN

namespace System
{
	// Inline memory operations
	inline bool memoryIsRead(const void* ptr, size_t bytes);
	inline bool memoryIsWrite(void* ptr, size_t bytes);
	inline void* memoryCopy(void* dst, const void* src, size_t bytes);
	inline void* memoryZero(void* dst, size_t bytes);

	// Inline ASCIIZ operations
	inline size_t asciizLength(const char* src);
	inline void asciizCopy(char* dst, const char* src, size_t size);
	inline void asciizConcat(char* dst, const char* src, size_t size);
	inline int asciizCompare(const char* lhs, const char* rhs);

	// Debug output
	bool debugOutput(const char* buffer);
	bool TEA_VARARGS debugFormat(const char* fmt, ...);
	void debugFlush(void);

	// Error reporting
	long errorGetLast(void);
	bool errorFormat(char* buffer, size_t size, long errcode);
	void errorReport(Window window, const char* caption, const char* reason);

	// Timer operations
	Timer timerGet(void);
	TimerElapsed timerElapsed(Timer before, Timer after);
	long timerMicroseconds(TimerElapsed elapsed);

	// Random number generator seed (from hires clock)
	int randomSeed(void);

	// Math operations
	inline long mathTruncate(double value);
	inline long mathRound(double value);
	inline long mathRoundUp(double value);
	inline long mathRoundDown(double value);
	inline double mathSin(double radians);
	inline double mathCos(double radians);
	inline double mathTan(double radians);
	inline void mathSinCos(float radians, float& sin, float& cos);
	inline void mathSinCos(double radians, double& sin, double& cos);
	inline double mathAsin(double x);
	inline double mathAcos(double x);
	inline double mathAtan(double x, double y);
	inline double mathPower(double x, double y);
	inline double mathSqrt(double value);
};

TEA_NAMESPACE_END

// Platform-specific includes (second time)
#ifdef WIN32
#include "TWin32.h"
#endif

#endif
